@@ -11,11 +11,11 @@ from biom.sparsemat import SparseMat, to_sparsemat, \
 
 __author__ = "Daniel McDonald"
 __copyright__ = "Copyright 2012, BIOM-Format Project"
-__credits__ = ["Daniel McDonald", "Jai Rideout", "Greg Caporaso", 
+__credits__ = ["Daniel McDonald", "Jai Ram Rideout", "Greg Caporaso", 
                "Jose Clemente", "Justin Kuczynski"]
 __license__ = "GPL"
 __url__ = "http://biom-format.org"
-__version__ = "1.0.0c"
+__version__ = "1.1.2"
 __maintainer__ = "Daniel McDonald"
 __email__ = "daniel.mcdonald@colorado.edu"
    
@@ -37,6 +37,21 @@ class SparseMatTests(TestCase):
         self.assertNotEqual(id(self.obj._index_rows), id(obs._index_rows))
         self.assertNotEqual(id(self.obj._index_cols), id(obs._index_cols))
 
+    def test_rebuildIndices(self):
+        """rebuild table indices"""
+        sm = SparseMat(3,4,enable_indices=False)
+        sm[0,0] = 10
+        sm[0,2] = 0
+        sm[1,3] = -5
+        sm[1,2] = 1
+        self.assertEqual(sm._index_rows, None)
+        self.assertEqual(sm._index_cols, None)
+        sm.rebuildIndices()
+        self.assertEqual(sm._index_rows, [set([(0,0)]),set([(1,3), (1,2)]),
+                                          set([])])
+        self.assertEqual(sm._index_cols, [set([(0,0)]), set([]),
+                                          set([(1,2)]), set([(1,3)])])
+
     def test_setitem(self):
         self.obj[(2,2)] = 10
         exp = sorted([((1,2),3),((5,2),6),((2,2),10)])
@@ -214,6 +229,22 @@ class SparseMatTests(TestCase):
         obs = self.obj.T
         self.assertEqual(obs, exp)
 
+    def test_get_size(self):
+        """test getting the number of nonzero elements"""
+        self.assertEqual(self.obj.size, 2)
+
+        # Test with setting an element explicitly to zero.
+        sm = SparseMat(2,4)
+        sm.update({(0,1):3,(1,2):7,(0,0):0})
+        self.assertEqual(sm.size, 2)
+
+        # Test with an empty matrix.
+        sm = SparseMat(2,4)
+        self.assertEqual(sm.size, 0)
+        sm = SparseMat(0,0)
+        self.assertEqual(sm.size, 0)
+
+
 class SupportTests(TestCase):
     def test_list_list_to_sparsemat(self):
         """convert [[row,col,value], ...] to sparsemat"""
